@@ -1,20 +1,25 @@
 /*
- * "$Id: printers.h 9621 2011-03-18 18:42:46Z mike $"
+ * "$Id: printers.h 12666 2015-05-25 19:38:09Z msweet $"
  *
- *   Printer definitions for the CUPS scheduler.
+ * Printer definitions for the CUPS scheduler.
  *
- *   Copyright 2007-2011 by Apple Inc.
- *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
+ * Copyright 2007-2013 by Apple Inc.
+ * Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
- *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ * These coded instructions, statements, and computer programs are the
+ * property of Apple Inc. and are protected by Federal copyright
+ * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ * which should have been included with this file.  If this file is
+ * file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 #ifdef HAVE_DNSSD
 #  include <dns_sd.h>
+#elif defined(HAVE_AVAHI)
+#  include <avahi-client/client.h>
+#  include <avahi-client/publish.h>
+#  include <avahi-common/error.h>
+#  include <avahi-common/thread-watch.h>
 #endif /* HAVE_DNSSD */
 #include <cups/pwg-private.h>
 
@@ -32,6 +37,20 @@ typedef struct
 } cupsd_quota_t;
 
 
+/*
+ * DNS-SD types to make the code cleaner/clearer...
+ */
+
+#ifdef HAVE_DNSSD
+typedef DNSServiceRef cupsd_srv_t;	/* Service reference */
+typedef TXTRecordRef cupsd_txt_t;	/* TXT record */
+
+#elif defined(HAVE_AVAHI)
+typedef AvahiEntryGroup *cupsd_srv_t;	/* Service reference */
+typedef AvahiStringList *cupsd_txt_t;	/* TXT record */
+#endif /* HAVE_DNSSD */
+
+
 /*
  * Printer/class information structure...
  */
@@ -44,9 +63,12 @@ struct cupsd_printer_s
 		*uuid,			/* Printer UUID */
 		*hostname,		/* Host printer resides on */
 		*name,			/* Printer name */
-		*location,		/* Location code */
+		*location,		/* Location string */
+		*geo_location,		/* Geographic location URI */
 		*make_model,		/* Make and model */
 		*info,			/* Description */
+		*organization,		/* Organization name */
+		*organizational_unit,	/* Organizational unit (department, etc.) */
 		*op_policy,		/* Operation policy name */
 		*error_policy;		/* Error policy */
   cupsd_policy_t *op_policy_ptr;	/* Pointer to operation policy */
@@ -58,12 +80,10 @@ struct cupsd_printer_s
   char		state_message[1024];	/* Printer state message */
   int		num_reasons;		/* Number of printer-state-reasons */
   char		*reasons[64];		/* printer-state-reasons strings */
-  time_t	state_time;		/* Time at this state */
+  time_t	config_time,		/* Time at this configuration */
+		state_time;		/* Time at this state */
   char		*job_sheets[2];		/* Banners/job sheets */
   cups_ptype_t	type;			/* Printer type (color, small, etc.) */
-  char		*browse_attrs;		/* Attributes sent with browse data */
-  time_t	browse_expire;		/* Expiration time for printer */
-  time_t	browse_time;		/* Last time update was sent/received */
   char		*device_uri;		/* Device URI */
   char		*sanitized_device_uri;	/* Sanitized device URI */
   char		*port_monitor;		/* Port monitor */
@@ -95,16 +115,17 @@ struct cupsd_printer_s
   time_t	marker_time;		/* Last time marker attributes were updated */
   _ppd_cache_t	*pc;			/* PPD cache and mapping data */
 
-#ifdef HAVE_DNSSD
+#if defined(HAVE_DNSSD) || defined(HAVE_AVAHI)
   char		*reg_name,		/* Name used for service registration */
-		*pdl,			/* pdl value for TXT record */
-		*ipp_txt,		/* IPP TXT record contents */
-		*printer_txt;		/* LPD TXT record contents */
-  int		ipp_len,		/* IPP TXT record length */
-		printer_len;		/* LPD TXT record length */
-  DNSServiceRef	ipp_ref,		/* Reference for _ipp._tcp,_cups */
-		printer_ref;		/* Reference for _printer._tcp */
-#endif /* HAVE_DNSSD */
+		*pdl;			/* pdl value for TXT record */
+  cupsd_srv_t	ipp_srv;		/* IPP service(s) */
+#  ifdef HAVE_DNSSD
+#    ifdef HAVE_SSL
+  cupsd_srv_t	ipps_srv;		/* IPPS service(s) */
+#    endif /* HAVE_SSL */
+  cupsd_srv_t	printer_srv;		/* LPD service */
+#  endif /* HAVE_DNSSD */
+#endif /* HAVE_DNSSD || HAVE_AVAHI */
 };
 
 
@@ -116,10 +137,8 @@ VAR ipp_t		*CommonData	VALUE(NULL);
 					/* Common printer object attrs */
 VAR cups_array_t	*CommonDefaults	VALUE(NULL);
 					/* Common -default option names */
-VAR cups_array_t	*Printers	VALUE(NULL),
+VAR cups_array_t	*Printers	VALUE(NULL);
 					/* Printer list */
-			*ImplicitPrinters VALUE(NULL);
-					/* Implicit class printers */
 VAR cupsd_printer_t	*DefaultPrinter	VALUE(NULL);
 					/* Default printer */
 VAR char		*DefaultPolicy	VALUE(NULL);
@@ -151,13 +170,15 @@ extern int		cupsdSetAuthInfoRequired(cupsd_printer_t *p,
 						 ipp_attribute_t *attr);
 extern void		cupsdSetDeviceURI(cupsd_printer_t *p, const char *uri);
 extern void		cupsdSetPrinterAttr(cupsd_printer_t *p,
-			                    const char *name, char *value);
+			                    const char *name,
+			                    const char *value);
 extern void		cupsdSetPrinterAttrs(cupsd_printer_t *p);
 extern int		cupsdSetPrinterReasons(cupsd_printer_t *p,
 			                       const char *s);
 extern void		cupsdSetPrinterState(cupsd_printer_t *p, ipp_pstate_t s,
 			                     int update);
-#define			cupsdStartPrinter(p,u) cupsdSetPrinterState((p), IPP_PRINTER_IDLE, (u))
+#define			cupsdStartPrinter(p,u) cupsdSetPrinterState((p), \
+						   IPP_PRINTER_IDLE, (u))
 extern void		cupsdStopPrinter(cupsd_printer_t *p, int update);
 extern int		cupsdUpdatePrinterPPD(cupsd_printer_t *p,
 			                      int num_keywords,
@@ -173,5 +194,5 @@ extern void		cupsdWritePrintcap(void);
 
 
 /*
- * End of "$Id: printers.h 9621 2011-03-18 18:42:46Z mike $".
+ * End of "$Id: printers.h 12666 2015-05-25 19:38:09Z msweet $".
  */
