@@ -1,16 +1,16 @@
 /*
- * "$Id: util.h 9755 2011-05-09 22:53:31Z mike $"
+ * "$Id: util.h 11558 2014-02-06 18:33:34Z msweet $"
  *
- *   Mini-daemon utility definitions for CUPS.
+ * Mini-daemon utility definitions for CUPS.
  *
- *   Copyright 2007-2011 by Apple Inc.
- *   Copyright 1997-2005 by Easy Software Products.
+ * Copyright 2007-2014 by Apple Inc.
+ * Copyright 1997-2005 by Easy Software Products.
  *
- *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ * These coded instructions, statements, and computer programs are the
+ * property of Apple Inc. and are protected by Federal copyright
+ * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ * which should have been included with this file.  If this file is
+ * file is missing or damaged, see the license at "http://www.cups.org/".
  */
 
 #ifndef _CUPSD_UTIL_H_
@@ -49,7 +49,7 @@ extern int		cupsdCompareNames(const char *s, const char *t);
 extern cups_array_t	*cupsdCreateStringsArray(const char *s);
 extern int		cupsdExec(const char *command, char **argv);
 extern cups_file_t	*cupsdPipeCommand(int *pid, const char *command,
-			                  char **argv, int user);
+			                  char **argv, uid_t user);
 extern void		cupsdSendIPPGroup(ipp_tag_t group_tag);
 extern void		cupsdSendIPPHeader(ipp_status_t status_code,
 			                   int request_id);
@@ -67,5 +67,5 @@ extern void		cupsdSendIPPTrailer(void);
 #endif /* !_CUPSD_UTIL_H_ */
 
 /*
- * End of "$Id: util.h 9755 2011-05-09 22:53:31Z mike $".
+ * End of "$Id: util.h 11558 2014-02-06 18:33:34Z msweet $".
  */
