@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# "$Id: 5.3-lpq.sh 8146 2008-11-19 19:50:56Z mike $"
+# "$Id: 5.3-lpq.sh 11398 2013-11-06 20:11:11Z msweet $"
 #
 #   Test the lpq command.
 #
@@ -17,7 +17,7 @@
 echo "LPQ Test"
 echo ""
 echo "    lpq -P Test1"
-../berkeley/lpq -P Test1 2>&1
+$VALGRIND ../berkeley/lpq -P Test1 2>&1
 if test $? != 0; then
 	echo "    FAILED"
 	exit 1
@@ -27,5 +27,5 @@ fi
 echo ""
 
 #
-# End of "$Id: 5.3-lpq.sh 8146 2008-11-19 19:50:56Z mike $".
+# End of "$Id: 5.3-lpq.sh 11398 2013-11-06 20:11:11Z msweet $".
 #
